package ru.myitschool.work.ui.login

import android.content.Context
import android.content.SharedPreferences
import android.os.Bundle
import android.text.Editable
import android.util.Log
import android.view.View
import android.widget.EditText
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.findNavController
import dagger.hilt.android.AndroidEntryPoint
import ru.myitschool.work.R
import ru.myitschool.work.api.ControllerAPI
import ru.myitschool.work.api.HandlersResultAPI
import ru.myitschool.work.api.entity.Employee
import ru.myitschool.work.core.MyConstants
import ru.myitschool.work.databinding.FragmentLoginBinding
import java.util.regex.Pattern

@AndroidEntryPoint
class LoginFragment : Fragment(R.layout.fragment_login), HandlersResultAPI {
    private var _binding: FragmentLoginBinding? = null
    private val binding: FragmentLoginBinding get() = _binding!!

    private val viewModel: LoginViewModel by viewModels()

    private var controllerAPI: ControllerAPI<LoginFragment>? = null // Работа с API
    private var settings: SharedPreferences? = null // Настройки приложения. Нужны для сохранения логина при входе.

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        _binding = FragmentLoginBinding.bind(view)

        // ТЗ:
        // >>> ✅ В пустом поле ввода должна отображаться подсказка, что требуется ввести пользователю.
        // >>> ✅ Если хотя бы одно из условий ниже соблюдено - кнопка должна быть неактивной:
        //         ✅ Поле ввода пустое.
        //         ✅ Количество символов менее 3-х.
        //         ✅ Логин начинается с цифры.
        //         ✅ Логин содержит символы, отличные от латинского алфавита и цифр.
        // >>> ✅ Поле ввода и кнопку должно быть видно при раскрытии клавиатуры.
        // >>> ✅ При нажатии на кнопку входа необходимо проверить, что данный пользователь существует
        //     с помощью запроса api/<LOGIN>/auth.
        // >>> ✅ В случае отсутствия логина или любой другой неполадки - необходимо вывести ошибку,
        //     пока пользователь не изменит текстовое поле или повторно не нажмёт на кнопку.
        // >>> ✅ После нажатия на кнопку - логин должен быть сохранён и при следующем открытии
        //     приложения экран авторизации не должен быть показан.
        // >>> ✅ После нажатия на кнопку - при нажатии стрелки назад - экран авторизации не
        //     должен быть показан повторно.
        // >>> ✅ Экран авторизации показывается только в случае, если пользователь не авторизован.
//        usernameEditText = findViewById<EditText>(R.id.username)
//        loginButton = findViewById<Button>(R.id.login)
//        errorTextView = findViewById<TextView>(R.id.error)

        controllerAPI = ControllerAPI<LoginFragment>()
        settings = this.requireActivity().getSharedPreferences(MyConstants.PREFS_FILE, Context.MODE_PRIVATE)


        // Этот код отвечает за прослушивание ввода логина. Нужен для 2 задания из ТЗ.
        // Он использует абстрактный класс UsernameEditTextListener, который просто сокращает TextWatcher.
        binding.username.addTextChangedListener(object : UsernameEditTextListener<EditText>(binding.username) {
            override fun onTextChanged(target: EditText, s: Editable) {
                val userLogin = s.toString()
                val pattern = Pattern.compile("[^a-zA-Z0-9]")
                val result = !(userLogin.isEmpty() || userLogin.length < 3 ||
                        Character.isDigit(userLogin[0]) ||
                        pattern.matcher(userLogin).find())

                binding.login.setEnabled(result)
                // ТЗ: Необходимо вывести ошибку, пока пользователь не изменит текстовое поле
                binding.error.visibility = View.GONE
            }
        })

        // Прослушиваем нажатие на кнопку `Вход`.
        binding.login.setOnClickListener {
            controllerAPI!!.verificationLogin(binding.username.text.toString(), this)
        }
    }

    override fun onDestroyView() {
        _binding = null
        super.onDestroyView()
    }

    override fun handlerVerificationLogin(login: String?, result: Boolean) {
        binding.error.visibility = View.GONE // Тз: необходимо вывести ошибку, пока пользователь
        // не изменит текстовое поле или повторно не нажмёт на кнопку.

        if (result) {
            // Этот код отвечает за сохранения факта входа.
            // Код копировал отсюда -> https://metanit.com/java/android/12.1.php

            val prefEditor = settings!!.edit()
            prefEditor.putString(MyConstants.PREF_LOGIN, login)
            prefEditor.apply()
            Log.d(
                "Test",
                "(LoginActivity) Проверка сохранения логина: " + settings!!.getString(
                    MyConstants.PREF_LOGIN,
                    "Не сохранилось"
                )
            )

            Log.d("Test", "(LoginActivity) Начало перехода на главную активность.")
            // findNavController().navigate(MainDestination)
            findNavController().popBackStack()

        } else {
            Log.d("Test", "(LoginActivity) Введён неправильный логин! login: $login")
            binding.error.visibility = View.VISIBLE
        }
    }

    override fun handlerGetEmployee(employee: Employee?) {
    }

    override fun handlerVisit(result: Int) {
    }
}